package com.wtkj.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.wtkj.entity.ApplyJoin;
import com.wtkj.entity.FileShare;
import com.wtkj.service.IApplyJoinService;
import com.wtkj.service.IQrcodeService;
import com.wtkj.wrapper.ApplyJoinWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * @author Blizzard
 * @create at 2023-09-19 17:25
 * @describe
 */
@RestController
@AllArgsConstructor
@RequestMapping("/qrcode")
@Api(value = "二维码模块", tags = "二维码模块")
public class QrCodeController {

	private final IQrcodeService qrcodeService;
	private final IApplyJoinService applyJoinService;

	/**
	 * 生成二维码
	 */
	@GetMapping("/create-dept-qrcode")
	@ApiOperation(value = "生成邀请用户至机构二维码", notes = "传入部门id")
	@ApiOperationSupport(order = 1)
	public R createQrcode(@RequestParam Long deptId) {
		return R.data(qrcodeService.createDeptQrcode(deptId));
	}

	/**
	 * 扫描二维码展现文字内容
	 */
	@GetMapping("/scan-invite-to-dept")
	@ApiOperation(value = "扫描邀请至部门二维码", notes = "")
	@ApiOperationSupport(order = 2)
	public R scanInviteToDept(@RequestParam String qrcodeId, @RequestParam Long createUser, @RequestParam Long deptId) {
		return R.data(qrcodeService.scanInviteToDept(qrcodeId, createUser, deptId));
	}

	/**
	 * 邀请至机构下
	 */
	@GetMapping("/invite-to-dept")
	@ApiOperation(value = "接受邀请加入机构", notes = "")
	@ApiOperationSupport(order = 3)
	public R inviteToDept(@RequestParam String qrcodeId, @RequestParam Long createUser, @RequestParam Long deptId) {
		return R.status(qrcodeService.inviteToDept(qrcodeId, createUser, deptId));
	}


	@PostMapping("/apply-to-dept")
	@ApiOperation(value = "申请加入机构", notes = "")
	@ApiOperationSupport(order = 4)
	public R applyToDept(@RequestBody @Valid ApplyJoin join) {
		return R.status(applyJoinService.saveOrUpdate(join));
	}

	/**
	 * 申请列表
	 */
	@GetMapping("/apply-page")
	@ApiOperation(value = "申请列表", notes = "传入当前登录用户顶级机构id")
	@ApiOperationSupport(order = 5)
	public R applyPage(@RequestParam Long topDept, Query query) {
		IPage<ApplyJoin> page = applyJoinService.page(Condition.getPage(query), new LambdaQueryWrapper<ApplyJoin>().eq(ApplyJoin::getDeptId, topDept));
		return R.data(ApplyJoinWrapper.build().pageVO(page));
	}

	/**
	 * 同意加入组织
	 */
	@GetMapping("/agree-apply")
	@ApiOperation(value = "同意加入机构", notes = "传入记录主键id")
	@ApiOperationSupport(order = 6)
	public R agreeJoinDept(@RequestParam Long id) {
		return R.status(applyJoinService.agreeApply(id));
	}

	/**
	 * 生成邀请至项目组二维码
	 */
	@GetMapping("/create-group-qrcode")
	@ApiOperation(value = "生成邀请至项目组二维码", notes = "")
	@ApiOperationSupport(order = 7)
	public R createGroupQrcode(@RequestParam Long projectId) {
		return R.data(qrcodeService.createGroupQrcode(projectId));
	}

	/**
	 * 扫描邀请至项目组二维码
	 */
	@GetMapping("/scan-invite-to-project")
	@ApiOperation(value = "扫描邀请至项目组二维码", notes = "")
	@ApiOperationSupport(order = 8)
	public R scanInviteToProject(@RequestParam String qrcodeId, @RequestParam Long createUser, @RequestParam Long projectId, @RequestParam Long topDept) {
		return R.data(R.data(qrcodeService.scanInviteToProject(qrcodeId, createUser, projectId, topDept)));
	}


	/**
	 * 邀请至项目组
	 */
	@GetMapping("/invite-to-project")
	@ApiOperation(value = "接受邀请至项目组", notes = "deptId:服务商的顶级机构id")
	@ApiOperationSupport(order = 9)
	public R inviteToProject(@RequestParam String qrcodeId, @RequestParam Long createUser, @RequestParam Long projectId, @RequestParam Long topDept) {
		return R.status(qrcodeService.inviteToProject(qrcodeId, createUser, projectId, topDept));
	}


	/**
	 * 生成一条分享记录
	 */
	@PostMapping("/save-share")
	@ApiOperationSupport(order = 10)
	@ApiOperation(value = "生成文件分享", notes = "ids:文件的")
	public R saveShare(@RequestBody FileShare share) {
		return R.data(qrcodeService.savaShare(share));
	}

	/**
	 * 查看分享
	 */
	@PostMapping("/get-share")
	@ApiOperationSupport(order = 11)
	@ApiOperation(value = "查看文件分享", notes = "传入二维码返回ID")
	public R getShare(@RequestParam Long id) {
		return R.data(qrcodeService.getShare(id));
	}


}
