package com.wtkj.handler;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.wtkj.config.KafkaConstant;
import com.wtkj.entity.FileAndFolder;
import com.wtkj.handler.es.Doc;
import com.wtkj.service.IFileAndFolderService;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.xcontent.XContentType;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;

import static com.wtkj.config.MagicValue.PROJECT_RESOURCE_INDEX;

/**
 * @author Blizzard
 * @create at 2023-09-27 11:31
 * @describe
 */
@Slf4j
@Component
public class EsFileHandler {

	@Resource
	private RestHighLevelClient client;
	@Resource
	private IFileAndFolderService fileAndFolderService;


	/**
	 * 文件上传转为图片以后通知消息  --->保存至es
	 */
	@KafkaListener(groupId = "parse-file-group", topics = KafkaConstant.PARSE_SUCCESS_MESSAGE, containerFactory = "ackContainerFactory")
	public void handSaveFileToEs(ConsumerRecord record, Acknowledgment ack) {
		try {
			String message = (String) record.value();
			log.info("收到文件解析成功消息，即将处理存入es逻辑:{}", message);

			JSONObject msg = (JSONObject) JSONObject.parse(message);
			Long fileId = msg.getLong("fileId");
			String content = msg.getString("content");
			Integer currentPage = msg.getInteger("currentPage");

			LambdaQueryWrapper<FileAndFolder> lqw = new LambdaQueryWrapper<>();
			lqw.eq(FileAndFolder::getBladeFileId, fileId);
			lqw.eq(FileAndFolder::getType, 1);
			List<FileAndFolder> files = fileAndFolderService.list(lqw);
			//避免重复处理
			if (!CollectionUtils.isEmpty(files) && files.size() == 1) {
				for (FileAndFolder file : files) {
					Long projectId = file.getProjectId();
					//使用自定义类作为数据存储容器
					Doc doc = new Doc();
					doc.setId(IdWorker.getIdStr());
					doc.setContent(content);
					doc.setFileId(String.valueOf(fileId));
					doc.setFileName(file.getTitle());
					doc.setCreateUser(String.valueOf(file.getCreateUser()));
					doc.setCurrentPage(currentPage);
					doc.setProjectId(String.valueOf(projectId));

					//创建文档
					IndexRequest indexRequest = new IndexRequest(PROJECT_RESOURCE_INDEX);
					indexRequest.id(doc.getId());
					indexRequest.timeout("1s");
					//将数据放入请求，将对象装为json格式
					indexRequest.source(JSONObject.toJSONString(doc), XContentType.JSON);
					client.index(indexRequest, RequestOptions.DEFAULT);
					//log.info("文档插入操作结果:" + putResponse.toString());

				}
			}
		} catch (Exception e) {
			log.error("文件存入es异常:" + e.getMessage());
		} finally {
			ack.acknowledge();
		}
	}
}
