package com.wtkj.handler;

import com.wtkj.entity.Project;
import com.wtkj.entity.ProjectStage;
import com.wtkj.entity.Task;
import com.wtkj.service.IProjectService;
import com.wtkj.service.IProjectStageService;
import com.wtkj.service.ITaskService;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.redis.cache.BladeRedis;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * @author Blizzard
 * @create at 2023-10-05 11:47
 * @describe 定时任务扫描任务状态
 */
@Configuration
@Slf4j
public class Schedule {

	@Autowired
	private ITaskService taskService;
	@Autowired
	private IProjectService projectService;
	@Autowired
	private IProjectStageService stageService;
	@Autowired
	private BladeRedis redis;


	@Scheduled(cron = "0 0 3 * * ?")
	private void task() {
		DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd"); // 设置日期格式
		String now = LocalDate.now().format(formatter);
		//今天过期的任务
		List<Task> tasks = taskService.getTodayToExpire(now);
		if (!CollectionUtils.isEmpty(tasks)) {
			tasks.forEach(task -> {
				task.setTaskStatus(6);
				task.setIsExpire(1);
				Long projectId = task.getProjectId();
				Project byId = projectService.getById(projectId);
				if (byId != null) {
					byId.setTodoTask(Math.max(byId.getTodoTask() - 1, 0));
					projectService.updateById(byId);
				}
			});
			taskService.updateBatchById(tasks);
		}
	}

	@Scheduled(cron = "0 0 4 * * ?")
	private void stageStatusTask() {
		DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd"); // 设置日期格式
		//1.是否有逾期任务
		List<Task> tasks = taskService.getByStatus(6);
		if (!CollectionUtils.isEmpty(tasks)) {
			int expireSize = tasks.size();
			tasks.forEach(task -> {
				Long stageId = task.getStageId();
				ProjectStage stage = stageService.getById(stageId);
				if (stage != null) {
					//有逾期 就为异常
					stage.setStageStatus(2);

					//失控  --->阶段下任务逾期数量 > 平均每天完成任务量
					//1.阶段周期
					List<LocalDate> dateList = redis.get("dateList::" + stageId);
					if (!CollectionUtils.isEmpty(dateList)) {
						int size = dateList.size();
						List<Task> allTask = taskService.listByStageId(stageId);
						if (!CollectionUtils.isEmpty(allTask)) {
							int taskAmount = allTask.size();
							// 理想每天任务完成量
							float avg = (float) taskAmount / size;
							if (expireSize > avg) {
								stage.setStageStatus(3);
							}
						}
					}
					stageService.updateById(stage);
				}
			});
		}
	}
}
