package com.wtkj.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.wtkj.entity.TaskContract;
import com.wtkj.entity.TaskContractPayLog;
import com.wtkj.mapper.TaskContractPayLogMapper;
import com.wtkj.service.ITaskContractPayLogService;
import com.wtkj.service.ITaskContractService;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author Blizzard
 * @create at 2023-10-12 15:19
 * @describe
 */
@Service
@AllArgsConstructor
public class TaskContractPayLogServiceImpl extends BaseServiceImpl<TaskContractPayLogMapper, TaskContractPayLog> implements ITaskContractPayLogService {

	private final ITaskContractService taskContractService;

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean submit(TaskContractPayLog payLog) {
		Long contractId = payLog.getContractId();
		TaskContract contract = taskContractService.getById(contractId);
		if (contract != null) {
			BigDecimal amount = contract.getAmount();

			BigDecimal payAmount = payLog.getPayAmount();
			//如果支付金额大于0
			if (payAmount.compareTo(BigDecimal.ZERO) > 0) {
				BigDecimal subtract = amount.subtract(payAmount);
				contract.setUnpaidAmount(subtract);
				taskContractService.updateById(contract);
			}
		}
		return this.saveOrUpdate(payLog);
	}

	@Override
	public boolean delete(List<Long> ids, Long contractId) {
		TaskContract contract = taskContractService.getById(contractId);
		if (contract != null) {
			BigDecimal unpaidAmount = contract.getUnpaidAmount();
			for (Long id : ids) {
				TaskContractPayLog payLog = this.getById(id);
				if (payLog != null) {
					BigDecimal payAmount = payLog.getPayAmount();
					unpaidAmount = unpaidAmount.add(payAmount);
				}
			}
			contract.setUnpaidAmount(unpaidAmount);
			taskContractService.updateById(contract);
		}
		return this.removeByIds(ids);
	}

	@Override
	public IPage<TaskContractPayLog> pageByContractId(Long contractId, IPage<TaskContractPayLog> page) {
		return page.setRecords(baseMapper.pageByContractId(contractId, page));
	}
}
