package com.wtkj.wrapper;

import com.wtkj.entity.Project;
import com.wtkj.entity.Task;
import com.wtkj.service.IProjectService;
import com.wtkj.vo.TaskVO;
import org.springblade.core.mp.support.BaseEntityWrapper;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.system.entity.DictBiz;
import org.springblade.system.feign.IDictBizClient;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author Blizzard
 * @create at 2023-10-13 09:51
 * @describe
 */
public class MyTaskWrapper extends BaseEntityWrapper<Task, TaskVO> {

	private static IUserClient userClient;

	private static IDictBizClient dictBizClient;
	private static IProjectService projectService;


	public static MyTaskWrapper build() {
		userClient = SpringUtil.getBean(IUserClient.class);
		dictBizClient = SpringUtil.getBean(IDictBizClient.class);
		projectService = SpringUtil.getBean(IProjectService.class);
		return new MyTaskWrapper();
	}

	@Override
	public TaskVO entityVO(Task entity) {
		TaskVO vo = Objects.requireNonNull(BeanUtil.copy(entity, TaskVO.class));
		Long projectId = entity.getProjectId();
		if (projectId != null) {
			Project byId = projectService.getById(projectId);
			if (byId != null) {
				vo.setProjectName(byId.getName());
			}
		}


		//任务执行人
		String executeUser = entity.getExecuteUser();
		if (executeUser != null) {
			List<String> execUsers = new ArrayList<>();
			List<Long> userIds = Func.toLongList(executeUser);
			userIds.forEach(userId -> {
				R<User> userR = userClient.userInfoById(userId);
				if (userR.isSuccess()) {
					execUsers.add(userR.getData().getName());
				}
			});
			vo.setExecuteUsers(execUsers);
		}

		//任务标签的数组
		String tags = entity.getTags();
		String category = entity.getCategory();
		List<String> dictBizs = new ArrayList<>();
		if (StringUtil.isNotBlank(tags)) {
			List<Long> intList = Func.toLongList(tags);
			intList.forEach(f -> {
				R<DictBiz> byId = dictBizClient.getById(f);
				if (byId.isSuccess()) {
					dictBizs.add(byId.getData().getDictValue());
				}
			});
		}
		if (StringUtil.isNotBlank(category)) {
			List<Long> intList = Func.toLongList(category);
			intList.forEach(f -> {
				R<DictBiz> byId = dictBizClient.getById(f);
				if (byId.isSuccess()) {
					dictBizs.add(byId.getData().getDictValue());
				}
			});
		}
		vo.setTagsArray(dictBizs);
		return vo;
	}
}
