package com.wtkj.wrapper;

import com.wtkj.entity.Project;
import com.wtkj.entity.Task;
import com.wtkj.service.IProjectService;
import com.wtkj.vo.TaskVO;
import org.springblade.core.mp.support.BaseEntityWrapper;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.BeanUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.SpringUtil;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.system.entity.Dept;
import org.springblade.system.entity.DictBiz;
import org.springblade.system.feign.IDictBizClient;
import org.springblade.system.feign.ISysClient;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author Blizzard
 * @create at 2023-10-09 15:52
 * @describe
 */
public class TaskPageWrapper extends BaseEntityWrapper<Task, TaskVO> {

	private static ISysClient sysClient;
	private static IUserClient userClient;
	private static IProjectService projectService;
	private static IDictBizClient dictBizClient;


	public static TaskPageWrapper build() {
		sysClient = SpringUtil.getBean(ISysClient.class);
		userClient = SpringUtil.getBean(IUserClient.class);
		projectService = SpringUtil.getBean(IProjectService.class);
		dictBizClient = SpringUtil.getBean(IDictBizClient.class);
		return new TaskPageWrapper();
	}

	@Override
	public TaskVO entityVO(Task entity) {
		TaskVO vo = Objects.requireNonNull(BeanUtil.copy(entity, TaskVO.class));

		Long projectId = entity.getProjectId();
		if (projectId != null) {
			Project byId = projectService.getById(projectId);
			if (byId != null) {
				vo.setProjectName(byId.getName());
			}
		}

		//协作单位
		Long orgDeptId = vo.getOrgDeptId();
		if (orgDeptId != null) {
			R<Dept> rpc = sysClient.getDept(orgDeptId);
			if (rpc.isSuccess()) {
				vo.setDispatchUintName(rpc.getData().getDeptName());
			}
		}

		//项目经理
		Long projectManager = vo.getProjectManager();
		if (projectManager != null) {
			R<User> userR = userClient.userInfoById(projectManager);
			if (userR.isSuccess()) {
				vo.setProjectManagerUser(userR.getData().getName());
			}
		}

		//任务标签的数组
		String tags = entity.getTags();
		String category = entity.getCategory();
		List<String> dictBizs = new ArrayList<>();
		if (StringUtil.isNotBlank(tags)) {
			List<Long> intList = Func.toLongList(tags);
			intList.forEach(f -> {
				R<DictBiz> byId = dictBizClient.getById(f);
				if (byId.isSuccess()) {
					dictBizs.add(byId.getData().getDictValue());
				}
			});
		}
		if (StringUtil.isNotBlank(category)) {
			List<Long> intList = Func.toLongList(category);
			intList.forEach(f -> {
				R<DictBiz> byId = dictBizClient.getById(f);
				if (byId.isSuccess()) {
					dictBizs.add(byId.getData().getDictValue());
				}
			});
		}
		vo.setTagsArray(dictBizs);


		return vo;
	}
}
